<?php

class Springbot_Main_Model_Entity_Order_OrderBuilder
{
    const type_amazon = 'amazon';
    const shipping_method = 'sbShipping';
    const payment_method = 'sbPayment';

    protected $data;
    protected $helper;

    /**
     * Springbot_Main_Model_Entity_Order_Builder constructor.
     * @param $data
     */
    public function __construct($data)
    {
        $this->data = $data;
        $this->helper = Mage::helper('springbot_main');
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function buildOrder()
    {
        $stockUpdated = false;
        try {

            // Calculate the totals from the data
            $itemTotals = $this->getItemTotals();
            $qtyArray = array();
            $total = $this->helper->fetch($this->data, array('order_total', 'Amount'));
            $subtotal = $this->getSubtotal($qtyArray);
            Mage::log('Order builder item totals: ' . $itemTotals, Zend_Log::DEBUG, 'sb.log');

            // Attempt to load the customer, use existing customer record if it exists
            $customer = Mage::getModel('springbot_main/entity_order_customerBuilder', $this->data)->createCustomer();
            $customer->cleanAllAddresses();

            // Retrieve miscellaneous data
            $storeId = $customer->getStoreId();
            $reservedOrderId = $this->reserveOrderId($storeId);
            $currencyCode = $this->getCurrencyCode();

            // Create the quote
            $quote = Mage::getModel('sales/quote')
                ->setStoreId($storeId)
                ->save();

            // Create the payment
            $orderPayment = Mage::getModel('sales/order_payment')
                ->setStoreId($storeId)
                ->setCustomerPaymentId(0)
                ->setMethod(self::payment_method)
                ->setPoNumber($this->helper->fetch($this->data, 'purchase_order_number'));

            // Create the order
            $order = Mage::getModel('sales/order')
                ->setIncrementId($reservedOrderId)
                ->setStoreId($storeId)
                ->setQuote($quote)
                ->setGlobalCurrencyCode($currencyCode)
                ->setBaseCurrencyCode($currencyCode)
                ->setStoreCurrencyCode($currencyCode)
                ->setOrderCurrencyCode($currencyCode)
                ->setCustomerEmail($customer->getEmail())
                ->setCustomerFirstname($customer->getFirstname())
                ->setCustomerLastname($customer->getLastname())
                ->setCustomerGroupId($customer->getGroupId())
                ->setCustomerIsGuest(0)
                ->setCustomer($customer)
                ->setAddresses()
                ->setSubtotal($subtotal)
                ->setBaseSubtotal($subtotal)
                ->setGrandTotal($total)
                ->setBaseGrandTotal($total)
                ->setTotalPaid($total)
                ->setBaseTotalPaid($total)
                ->setShippingAmount($itemTotals['shipping'])
                ->setBaseShippingAmount($itemTotals['shipping'])
                ->setShippingTaxAmount($itemTotals['shipping_tax'])
                ->setBaseShippingTaxAmount($itemTotals['shipping_tax'])
                ->setTaxAmount($itemTotals['tax'])
                ->setBaseTaxAmount($itemTotals['tax']);
            $order->setPayment($orderPayment);

            // Set the billing and shipping addresses
            $this->setAddresses($customer, $order, $storeId);

            // Create the Springbot marketplace order
            $mpOrder = $this->createMarketplaceOrder($order);

            // Add the line items to the order
            foreach ($this->helper->fetch($this->data, array('order_items'), null, true) as $itemArray) {
                $product = $this->parseProduct($itemArray);
                $item = Mage::getModel('springbot_main/entity_order_itemBuilder', $itemArray)->makeOrderItem($product);
                $order->addItem($item);
                $qtyArray = $this->addToStockQtyArray($qtyArray, $item);
            }

            // Update the stock levels accordingly
            $stock = Mage::getSingleton('cataloginventory/stock');
            $transaction = Mage::getModel('core/resource_transaction');
            $stock->registerProductsSale($qtyArray);
            $stockUpdated = true;
            $transaction->addObject($order)
                ->addObject($mpOrder)
                ->addCommitCallback(array($order, 'place'))
                ->addCommitCallback(array($order, 'save'))
                ->save();

            // Verify the order was created successfully
            if ($orderId = $order->getId()) {
                $mpOrder->setOrderId($orderId)->save();
                return $order;
            }
            throw new Exception('Failed to save order');

        } catch (Zend_Db_Statement_Exception $e) {
            if ($stockUpdated) {
                $stock->revertProductsSale($qtyArray);
            }
            throw new Exception("Order ID already exists {$e->getMessage()}", 409);
        }
    }

    /**
     * @param $order
     * @return mixed
     */
    private function createMarketplaceOrder($order)
    {
        Mage::log("Making order for {$this->helper->fetch($this->data, 'amazon_order_id')}", Zend_Log::DEBUG, 'sb.log');
        $mpOrder = Mage::getModel('springbot_main/marketplace_order');
        $mpOrder->setData(array(
            'increment_id'     => $order->getIncrementId(),
            'remote_order_id'  => $this->helper->fetch($this->data, 'amazon_order_id'),
            'marketplace_type' => self::type_amazon,
            'created_at'       => time()
        ));
        Mage::log($mpOrder->getData(), Zend_Log::DEBUG, 'sb.log');
        return $mpOrder;
    }

    /**
     * @param $customer
     * @param $order
     * @param $storeId
     * @return mixed
     */
    private function setAddresses($customer, $order, $storeId)
    {
        // Create the billing address
        $billing = $customer->getDefaultBillingAddress();
        $billingAddress = Mage::getModel('sales/order_address')
            ->setStoreId($storeId)
            ->setAddressType(Mage_Sales_Model_Quote_Address::TYPE_BILLING)
            ->setCustomerId($customer->getId())
            ->setCustomerAddressId($customer->getDefaultBilling())
            ->setCustomerAddress_id($billing->getEntityId())
            ->setPrefix($billing->getPrefix())
            ->setFirstname($billing->getFirstname())
            ->setMiddlename($billing->getMiddlename())
            ->setLastname($billing->getLastname())
            ->setSuffix($billing->getSuffix())
            ->setCompany($billing->getCompany())
            ->setStreet($billing->getStreet())
            ->setCity($billing->getCity())
            ->setCountry_id($billing->getCountryId())
            ->setRegion($billing->getRegion())
            ->setRegion_id($billing->getRegionId())
            ->setPostcode($billing->getPostcode())
            ->setTelephone($billing->getTelephone())
            ->setFax($billing->getFax())
            ->setVatId($billing->getVatId());

        // Create the shipping address
        $shipping = $customer->getDefaultShippingAddress();
        $shippingAddress = Mage::getModel('sales/order_address')
            ->setStoreId($storeId)
            ->setAddressType(Mage_Sales_Model_Quote_Address::TYPE_SHIPPING)
            ->setCustomerId($customer->getId())
            ->setCustomerAddressId($customer->getDefaultShipping())
            ->setCustomer_address_id($shipping->getEntityId())
            ->setPrefix($shipping->getPrefix())
            ->setFirstname($shipping->getFirstname())
            ->setMiddlename($shipping->getMiddlename())
            ->setLastname($shipping->getLastname())
            ->setSuffix($shipping->getSuffix())
            ->setCompany($shipping->getCompany())
            ->setStreet($shipping->getStreet())
            ->setCity($shipping->getCity())
            ->setCountry_id($shipping->getCountryId())
            ->setRegion($shipping->getRegion())
            ->setRegion_id($shipping->getRegionId())
            ->setPostcode($shipping->getPostcode())
            ->setTelephone($shipping->getTelephone())
            ->setFax($shipping->getFax())
            ->setVatId($billing->getVatId());

        // Set the addresses on the order
        $order->setBillingAddress($billingAddress)
            ->setShippingAddress($shippingAddress)
            ->setshipping_method(self::shipping_method)
            ->setShippingDescription($this->helper->fetch($this->data, array('shipment_service_level_category')));

        return $order;
    }

    /**
     * @return mixed
     */
    private function getCurrencyCode()
    {
        if ($value = $this->helper->fetch($this->data, array('order_total', 'CurrencyCode'))) {
            return $value;
        }
        else {
            return Mage::app()->getBaseCurrencyCode();
        }
    }

    /**
     * @param $storeId
     * @return mixed
     */
    private function reserveOrderId($storeId)
    {
        return Mage::getSingleton('eav/config')
            ->getEntityType('order')
            ->fetchNewIncrementId($storeId);
    }

    /**
     * @param $itemTotals
     * @return mixed
     */
    private function getSubtotal($itemTotals)
    {
        return $itemTotals['item_price'] + $itemTotals['tax'];
    }

    /**
     * @return array
     */
    private function getItemTotals()
    {
        $shipping = 0;
        $shippingTax = 0;
        $tax = 0;
        $itemPrice = 0;
        foreach ($this->helper->fetch($this->data, 'order_items') as $item) {
            $shipping += $this->helper->fetch($item, array('shipping_price', 'Amount'));
            $shippingTax += $this->helper->fetch($item, array('shipping_tax', 'Amount'));
            $tax += $this->helper->fetch($item, array('item_tax', 'Amount'));
            $itemPrice += $this->helper->fetch($item, array('item_price', 'Amount'));
        }
        $itemTotals = array(
            'shipping'     => $shipping,
            'shipping_tax' => $shippingTax,
            'tax'          => $tax,
            'item_price'   => $itemPrice
        );
        Mage::log($itemTotals, Zend_Log::DEBUG, 'sb.log');
        return $itemTotals;
    }

    /**
     * Prepare array with information about used product qty and product stock item
     * result is:
     * array(
     *  $productId  => array(
     *      'qty'   => $qty,
     *      'item'  => $stockItems|null
     *  )
     * )
     *
     * @param array $qtyArray
     * @param $item
     * @return mixed
     */
    private function addToStockQtyArray($qtyArray, $item)
    {
        $productId = $item->getProductId();
        $qty = $item->getQtyOrdered();
        $stockItem = $item->getProduct()->getStockItem();
        Mage::log("Adding {$qty} of product {$productId}", Zend_Log::DEBUG, 'sb.log');
        if ($qty && $productId) {
            if (isset($qtyArray[$productId])) {
                $qtyArray[$productId]['qty'] += $qty;
            }
            else {
                $qtyItem = array(
                    'qty'  => $qty,
                    'item' => $stockItem
                );
                $qtyArray[$productId] = $qtyItem;
            }
        }
        return $qtyArray;
    }

    /**
     * @param $item
     * @return mixed
     * @throws Exception
     */
    public function parseProduct($item)
    {
        $id = $this->helper->fetch($item, 'product_id');
        $product = Mage::getModel('catalog/product')->load($id);
        if (is_null($product->getId())) {
            throw new Exception("Could not find product where id = {$id}", 409);
        }
        return $product;
    }

}