<?php

class Springbot_Main_Model_Store extends Mage_Core_Model_Abstract
{

    const app_url = 'https://app.springbot.com';

    private $entity_types = array(
        'orders'         => 'springbot_main/entity_order',
        'carts'          => 'springbot_main/entity_cart',
        'subscribers'    => 'springbot_main/entity_subscriber',
        'customers'      => 'springbot_main/entity_customer',
        'guests'         => 'springbot_main/entity_guest',
        'attribute-sets' => 'springbot_main/entity_attributeSet',
        'rules'          => 'springbot_main/entity_rule',
        'products'       => 'springbot_main/entity_product',
        'inventory'      => 'springbot_main/entity_inventory',
        'categories'     => 'springbot_main/entity_category',
    );

    /**
     * @param $email
     * @param $password
     * @param null $apiToken
     * @return bool
     * @throws Exception
     */
    public function registerStores($email = null, $password = null, $apiToken = null)
    {
        if (is_null($email) && is_null($password) && is_null($apiToken)) {
            if (!($apiToken = Mage::getStoreConfig("springbot/main/api_token"))) {
                throw new Exception('No credentials set to register stores');
            }
        }

        $auth = Mage::getModel('springbot_router/auth');
        $stores = $this->getAllStores();
        $pluginVersion = Mage::getConfig()->getNode('modules/Springbot_Main/version');

        $storesData = array(
            'stores'         => $this->storesToArray($stores),
            'plugin_version' => (string)$pluginVersion,
            'platform'       => 'magento1',
            'access_token'   => $auth->getAccessToken(),
            'category_filter'=> false,
            'root_category_filter'=> false,
            'credentials'    => array(
                'email'     => $email,
                'password'  => $password,
                'api_token' => $apiToken
            )
        );

        $api = Mage::getModel('springbot_main/api');
        $url = trim(Mage::getStoreConfig("springbot/main/api_url"), '/') . '/magento2/create.json';
        $response = $api->post($url, json_encode($storesData));

        $decoded = json_decode($response->getBody(), true);
        if (!$decoded) {
            throw new Exception('Invalid response from API.');
        }
        if (!isset($decoded['security_token'])) {
            throw new Exception('Invalid username or password');
        }
        Mage::getConfig()->saveConfig("springbot/main/api_token", $decoded['security_token']);

        $rewrite = Mage::getModel('springbot_main/rewrite');
        foreach ($decoded['stores'] as $guid => $storeArray) {
            if (!isset($storeArray['json_data'])) {
                throw new Exception("json_data not set in JSON response during registration");
            }
            $localId = $storeArray['json_data']['store_id'];
            if (!is_numeric($localId)) {
                throw new Exception("Invalid local store id during registration: {$localId}");
            }
            $springbotId = $storeArray['springbot_store_id'];
            Mage::getConfig()->saveConfig("springbot/main/store_guid_{$localId}", $guid);
            Mage::getConfig()->saveConfig("springbot/main/store_id_{$localId}", $springbotId);
            if (is_object($stores[$localId])) {
                $rewrite->createRewrite($stores[$localId], "springbot/{$localId}", 'i',
                    self::app_url . "/i/{$springbotId}");
            }
        }

        $cache = Mage::getModel('springbot_main/cache');
        $cache->clearCache();

        return true;
    }

    /**
     * @return bool
     */
    public function isRegistered()
    {
        if (Mage::getStoreConfig("springbot/main/api_token")) {
            return true;
        }
        return false;
    }

    /**
     * @return bool
     */
    public function isConfigured()
    {
        if (Mage::getStoreConfig("admin/security/extensions_compatibility_mode")) {
            return true;
        }
        return false;
    }

    /**
     * @return class Springbot_Main_Model_Api
     */
    public function api()
    {
        return Mage::getModel('springbot_main/api');
    }

    /**
     * @param $store
     * @return array
     */
    public function storeToArray($store)
    {
        $localId = $store->getId();
        return array(
            "store_id"            => $localId,
            "guid"                => $this->getGuid($store),
            "name"                => $store->getName(),
            "store_name"          => $store->getName(),
            "store_code"          => $store->getCode(),
            "url"                 => $store->getBaseUrl('link'),
            "store_url"           => $store->getBaseUrl('link'),
            "store_active"        => $store->getIsActive(),
            "secure_url"          => $store->getBaseUrl('link', true),
            "media_url"           => $store->getBaseUrl('media'),
            "web_id"              => $store->getWebsiteId(),
            'store_mail_address'  => Mage::getStoreConfig('general/store_information/address'),
            'store_custsrv_email' => Mage::getStoreConfig('trans_email/ident_support/email'),
            'logo_src'            => Mage::getStoreConfig('design/header/logo_src'),
            'logo_alt_tag'        => Mage::getStoreConfig('design/header/logo_alt'),
            'store_statuses'      => Mage::getModel('sales/order')->getConfig()->getStatuses(),
        );
    }

    /**
     * @return Springbot_Main_Model_Api
     * @throws Exception
     */
    public function getApi()
    {
        $apiToken = Mage::getStoreConfig("springbot/main/api_token");
        if (!$apiToken) {
            throw new Exception('API Token not yet set for instance. Owner may need to enter credentials');
        }
        return Mage::getModel('springbot_main/api');
    }

    /**
     * @param $store
     * @return array
     */
    public function getCounts($store)
    {
        $counts = array();
        foreach ($this->entity_types as $type => $modelPath) {
            $model = Mage::getModel($modelPath, array('store_id' => $store->getId()));
            $counts[] = array(
                'entity_type' => $type,
                'count'       => $model->getCount($store->getId())
            );
        }
        return $counts;
    }

    /**
     * Return an array of all stores on this instance
     * @return array
     */
    public function getAllStores()
    {
        $stores = array();
        foreach (Mage::app()->getWebsites() as $website) {
            foreach ($website->getGroups() as $group) {
                $views = $group->getStores();
                foreach ($views as $view) {
                    $stores[$view->getStoreId()] = $view;
                }
            }
        }
        return $stores;
    }


    /**
     * @param $store
     * @return string
     */
    private function getGuid($store)
    {
        $localId = $store->getId();
        $guid = Mage::getStoreConfig("springbot/main/store_guid_{$localId}");
        if (!$guid) {
            $uniqueString = sha1($store->getBaseUrl() . $localId);
            $guid = substr($uniqueString, 0, 8) . '-'
                . substr($uniqueString, 8, 4) . '-'
                . substr($uniqueString, 12, 4) . '-'
                . substr($uniqueString, 16, 4) . '-'
                . substr($uniqueString, 20, 12);
        }
        return $guid;
    }

    /**
     * Iterate through all stores and convert them to an array
     * @param $stores
     * @return array
     */
    private function storesToArray($stores)
    {
        $storesArray = array();
        foreach ($stores as $store) {
            $storesArray[] = $this->storeToArray($store);
        }
        return $storesArray;
    }

}
