<?php

class Springbot_Queue_Model_Manager extends Mage_Core_Model_Abstract
{

    protected function _construct()
    {
        $this->_init('springbot_queue/manager');
    }

    /**
     * Enqueue a new job to be run
     *
     * @param $class
     * @param $method
     * @param array $args
     * @param string $queue
     * @param int $priority
     */
    public function enqueue($class, $method, array $args = array(), $queue = 'default', $priority = 1)
    {
        try {
            $jsonArgs = json_encode($args);
            $hash = sha1($class . $method . $jsonArgs);
            $job = Mage::getModel('springbot_queue/job')->setData(array(
                'class'      => $class,
                'method'     => $method,
                'priority'   => $priority,
                'queue'      => $queue,
                'args'       => $jsonArgs,
                'hash'       => $hash,
                'created_at' => time()
            ));
            $job->save();
        } catch (Exception $e) {
            Mage::log($e->getMessage(), Zend_Log::DEBUG, 'sb.log');
        }
    }

    /**
     * Process the next jobs from the queue. This is different from runNextJob() in that it will run up to the
     * max-jobs-per-request value.
     */
    public function process()
    {
        return $this->runNextJobs();
    }

    /**
     * Run the next job in the queue based on priority and next_run_at value
     *
     * @param int $max The maximum number of jobs to run
     * @return bool
     */
    public function runNextJobs($max = 2)
    {
        $jobs = $this->getJobs($max)
            ->addFieldToFilter('attempts', array('lt' => 10));
        foreach ($jobs as $job) {
            try {
                $job->preRun();
                $job->run();
                $job->delete();
            } catch (Exception $e) {
                $job->postRun($e->getMessage());
                $job->save();
                return false;
            }
        }
        return true;
    }

    /**
     * Return an array of jobs based on the page supplied
     *
     * @param int $pageSize
     * @param int $page
     * @param string $sortBy
     * @return Springbot_Queue_Model_Resource_Job_Collection
     */
    public function getJobs($pageSize = 10, $page = 1, $sortBy = 'id')
    {
        return Mage::getModel('springbot_queue/job')
            ->getCollection()
            ->setPageSize($pageSize)
            ->setCurPage($page)
            ->setOrder($sortBy);
    }

    /**
     * Return the number of jobs currently scheduled to run based on attempts and next_run_at
     *
     * @return int
     */
    public function getJobCount()
    {
        return Mage::getModel('springbot_queue/job')
            ->getCollection()
            ->addFieldToFilter('attempts', array('lt' => 10))
            ->getSize();
    }

    /**
     * Delete all jobs currently in the queue
     */
    public function clearJobs()
    {
        $collection = Mage::getModel('springbot_queue/job')
            ->getCollection();
        foreach ($collection as $job) {
            $job->delete();
        }
    }

    /**
     * Delete a job by its ID
     *
     * @param $jobId
     * @return boolean
     */
    public function deleteJob($jobId)
    {
        return Mage::getModel('springbot_queue/job')
            ->load($jobId)
            ->delete();
    }

}